@@ -1,27 +1,28 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2014, Florian Kirsch,
+// Copyright (C) 2002-2022, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // channelManager.cpp
 //
 
 #include "opencsgConfig.h"
+#include "channelManager.h"
+
 #include <GL/glew.h>
 #ifdef _WIN32
 #include <GL/wglew.h>
@@ -29,7 +30,6 @@
 #include <GL/glxew.h>
 #endif
 
-#include "channelManager.h"
 #include "context.h"
 #include "offscreenBuffer.h"
 #include "openglHelper.h"
@@ -42,8 +42,6 @@ namespace OpenCSG {
 
     namespace {
 
-        GLint FaceOrientation = GL_CCW;
-
         int nextPow2(int value) {
             if(value <= 0) { return 0; }
             int result = 1;
@@ -71,7 +69,7 @@ namespace OpenCSG {
             int mSecondMax;
             int mCounter;
         public:
-            MaximumMemorizer() : mMax(0), mSecondMax(-1) { }
+            MaximumMemorizer() : mMax(0), mSecondMax(-1), mCounter(0) { }
             void newValue(int v) {
                 if (v>=mMax) {
                     mMax = v;
@@ -97,6 +95,10 @@ namespace OpenCSG {
 
     ChannelManager::ChannelManager()
       : mOffscreenBuffer(0)
+      , mInOffscreenBuffer(false)
+      , mFaceOrientation(GL_CCW)
+      , mCurrentChannel(NoChannel)
+      , mOccupiedChannels(NoChannel)
     {
         glPushAttrib(GL_ALL_ATTRIB_BITS);
         glDisable(GL_LIGHTING);
@@ -105,11 +107,13 @@ namespace OpenCSG {
         if (GLEW_ARB_texture_rectangle || GLEW_EXT_texture_rectangle || GLEW_NV_texture_rectangle)
             glDisable(GL_TEXTURE_RECTANGLE_ARB);
         glDisable(GL_TEXTURE_3D); // OpenGL 1.2 - take this as given
-        if (GL_ARB_texture_cube_map)
+        if (GLEW_ARB_texture_cube_map)
             glDisable(GL_TEXTURE_CUBE_MAP_ARB);
         glDisable(GL_BLEND);
 
-        glGetIntegerv(GL_FRONT_FACE, &FaceOrientation);
+        GLint faceOrientation;
+        glGetIntegerv(GL_FRONT_FACE, &faceOrientation);
+        mFaceOrientation = static_cast<GLenum>(faceOrientation);
 
         glGetFloatv(GL_MODELVIEW_MATRIX, OpenGL::modelview);
         glGetFloatv(GL_PROJECTION_MATRIX, OpenGL::projection);
@@ -172,12 +176,18 @@ namespace OpenCSG {
 
         mOffscreenBuffer = OpenGL::getOffscreenBuffer(newOffscreenType);
 
-        if (!mOffscreenBuffer) {
+        if (!mOffscreenBuffer)
+        {
             // Creating the offscreen buffer failed, maybe the OpenGL extension
             // for the specific offscreen buffer type is not supported
             return false;
         }
 
+        if (!mOffscreenBuffer->ReadCurrent())
+        {
+            return false;
+        }
+
         const int dx = OpenGL::canvasPos[2] - OpenGL::canvasPos[0];
         const int dy = OpenGL::canvasPos[3] - OpenGL::canvasPos[1];
 
@@ -221,7 +231,8 @@ namespace OpenCSG {
 
         bool rebuild = false;
 
-        if (!mOffscreenBuffer->IsInitialized()) {
+        if (!mOffscreenBuffer->IsInitialized())
+        {
             if (!mOffscreenBuffer->Initialize(sizeX.getMax(), sizeY.getMax(), true, false)) {
                 // Initializing the offscreen buffer failed, maybe the OpenGL extension
                 // for the specific offscreen buffer type is not supported
@@ -230,18 +241,18 @@ namespace OpenCSG {
             rebuild = true;
         }
         // tx == ty == 0 happens if the window is minimized, in this case don't touch a thing
-        else if (tx != 0 && ty != 0) {
-            if (   mOffscreenBuffer->GetWidth() != sizeX.getMax()
-                || mOffscreenBuffer->GetHeight() != sizeY.getMax()
-            ) {
-                if (!mOffscreenBuffer->Resize(sizeX.getMax(), sizeY.getMax())) {
-                    // Resizing the offscreen buffer failed, maybe the OpenGL extension
-                    // for the specific offscreen buffer type is not supported. More 
-                    // likely this is a programming error in Resize(). 
-                    return false;
-                }
-                rebuild = true;
+        else if (tx != 0 && ty != 0 &&
+                    (   mOffscreenBuffer->GetWidth() != sizeX.getMax()
+                     || mOffscreenBuffer->GetHeight() != sizeY.getMax()
+                )   )
+        {
+            if (!mOffscreenBuffer->Resize(sizeX.getMax(), sizeY.getMax())) {
+                // Resizing the offscreen buffer failed, maybe the OpenGL extension
+                // for the specific offscreen buffer type is not supported. More
+                // likely this is a programming error in Resize().
+                return false;
             }
+            rebuild = true;
         }
 
         if (rebuild) {
@@ -294,7 +305,7 @@ namespace OpenCSG {
         if (!mInOffscreenBuffer) {
             mOffscreenBuffer->BeginCapture();
             if (mOffscreenBuffer->haveSeparateContext()) {
-                glFrontFace(FaceOrientation);
+                glFrontFace(mFaceOrientation);
             }
 
             mInOffscreenBuffer = true;
@@ -380,27 +391,31 @@ namespace OpenCSG {
     }
 
 
-    void ChannelManager::setupProjectiveTexture() {
-        static float splane[4] = { 1.0f, 0.0f, 0.0f, 0.0f };
-        static float tplane[4] = { 0.0f, 1.0f, 0.0f, 0.0f };
-        static float rplane[4] = { 0.0f, 0.0f, 1.0f, 0.0f };
-        static float qplane[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
+    void ChannelManager::setupProjectiveTexture(bool fixedFunction)
+    {
+        static const float splane[4] = { 1.0f, 0.0f, 0.0f, 0.0f };
+        static const float tplane[4] = { 0.0f, 1.0f, 0.0f, 0.0f };
+        static const float rplane[4] = { 0.0f, 0.0f, 1.0f, 0.0f };
+        static const float qplane[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
 
         mOffscreenBuffer->Bind();
         mOffscreenBuffer->EnableTextureTarget();
 
-        glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-        glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-        glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-        glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-        glTexGenfv(GL_S, GL_EYE_PLANE, splane);
-        glTexGenfv(GL_T, GL_EYE_PLANE, tplane);
-        glTexGenfv(GL_R, GL_EYE_PLANE, rplane);
-        glTexGenfv(GL_Q, GL_EYE_PLANE, qplane);
-        glEnable(GL_TEXTURE_GEN_S);
-        glEnable(GL_TEXTURE_GEN_T);
-        glEnable(GL_TEXTURE_GEN_R);
-        glEnable(GL_TEXTURE_GEN_Q);
+        if (fixedFunction)
+        {
+            glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+            glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+            glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+            glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+            glTexGenfv(GL_S, GL_EYE_PLANE, splane);
+            glTexGenfv(GL_T, GL_EYE_PLANE, tplane);
+            glTexGenfv(GL_R, GL_EYE_PLANE, rplane);
+            glTexGenfv(GL_Q, GL_EYE_PLANE, qplane);
+            glEnable(GL_TEXTURE_GEN_S);
+            glEnable(GL_TEXTURE_GEN_T);
+            glEnable(GL_TEXTURE_GEN_R);
+            glEnable(GL_TEXTURE_GEN_Q);
+        }
 
         glMatrixMode(GL_TEXTURE);
 
@@ -426,25 +441,30 @@ namespace OpenCSG {
             factorY /= static_cast<float>(mOffscreenBuffer->GetHeight());
         }
 
-        float   texCorrect[16] = { factorX, 0.0f, 0.0f, 0.0f, 
-                                   0.0f, factorY, 0.0f, 0.0f, 
+        float   texCorrect[16] = { factorX, 0.0f, 0.0f, 0.0f,
+                                   0.0f, factorY, 0.0f, 0.0f,
                                    0.0f,    0.0f, 1.0f, 0.0f,
                                    0.0f,    0.0f, 0.0f, 1.0f };
-        
-        static float p2ndc[16] = { 0.5f, 0.0f, 0.0f, 0.0f, 
-                                   0.0f, 0.5f, 0.0f, 0.0f, 
-                                   0.0f, 0.0f, 0.5f, 0.0f, 
-                                   0.5f, 0.5f, 0.5f, 1.0f };
+
+        static const float p2ndc[16] = { 0.5f, 0.0f, 0.0f, 0.0f,
+                                         0.0f, 0.5f, 0.0f, 0.0f,
+                                         0.0f, 0.0f, 0.5f, 0.0f,
+                                         0.5f, 0.5f, 0.5f, 1.0f };
         glPushMatrix();
         glLoadMatrixf(texCorrect);
         glMultMatrixf(p2ndc);
-        glMultMatrixf(OpenGL::projection);
-        glMultMatrixf(OpenGL::modelview);
+        if (fixedFunction)
+        {
+            glMultMatrixf(OpenGL::projection);
+            glMultMatrixf(OpenGL::modelview);
+        }
         glMatrixMode(GL_MODELVIEW);
     }
 
-    void ChannelManager::resetProjectiveTexture() {
-        if (!mOffscreenBuffer->haveSeparateContext()) {
+    void ChannelManager::resetProjectiveTexture(bool fixedFunction)
+    {
+        if (fixedFunction && !mOffscreenBuffer->haveSeparateContext())
+        {
             glDisable(GL_TEXTURE_GEN_S);
             glDisable(GL_TEXTURE_GEN_T);
             glDisable(GL_TEXTURE_GEN_R);
