@@ -1,21 +1,20 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2011, Florian Kirsch,
+// Copyright (C) 2002-2022, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // channelManager.h 
@@ -27,6 +26,7 @@
 #define __OpenCSG__channel_manager_h__
 
 #include "opencsgConfig.h"
+#include <GL/glew.h>
 #include <utility>
 #include <vector>
 
@@ -39,7 +39,7 @@ namespace OpenCSG {
     class Primitive;
 
     enum Channel {
-        NoChannel = 0, Alpha = 1, Red = 2, Green = 4, Blue = 8
+        NoChannel = 0, Alpha = 1, Red = 2, Green = 4, Blue = 8, AllChannels = 15
     };
 
     class ChannelManager {
@@ -59,7 +59,7 @@ namespace OpenCSG {
         Channel find() const;
         /// allocates a new channel for temporary calculation of visibility
         /// information. return NoChannel if no free channel was found.
-        Channel request();
+        virtual Channel request();
         /// returns the currently used channel for visibility calculation
         Channel current() const;
         /// returns channels that currently contain visibility information
@@ -77,19 +77,27 @@ namespace OpenCSG {
         /// setups texture stuff that makes the size of the offscreen buffer
         /// and the size of the main canvas correspond. activates texture
         /// containing the content of the offscreen buffer
-        void setupProjectiveTexture();
+        void setupProjectiveTexture(bool fixedFunction);
         /// undoes texture settings
-        void resetProjectiveTexture();
+        void resetProjectiveTexture(bool fixedFunction);
         /// activate texenv settings such that information in channel is
         /// moved into alpha, to allow alpha testing of the channel.
         static void setupTexEnv(Channel channel);
 
+    protected:
+        bool isRectangularTexture() const;
+
     private:
+        ChannelManager(const ChannelManager&);
+        ChannelManager& operator=(const ChannelManager&);
 
-        static OpenGL::OffscreenBuffer* gOffscreenBuffer;
         static bool gInUse;
 
+        OpenGL::OffscreenBuffer* mOffscreenBuffer;
         bool mInOffscreenBuffer;
+        GLenum mFaceOrientation;
+
+    protected:
         Channel mCurrentChannel;
         int mOccupiedChannels;
     };
