@@ -1,20 +1,19 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2006-2011, Florian Kirsch
+// Copyright (C) 2006-2022, Florian Kirsch
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // frameBufferObject.h 
@@ -39,6 +38,9 @@ namespace OpenCSG {
             FrameBufferObject();
             virtual ~FrameBufferObject();
 
+            /// Reads the currently bound FBO.
+            virtual bool ReadCurrent();
+
             /// Call this once before use. shareObjects and copyContext are
             /// ignored, since there are no separate contexts for frame
             /// buffer objects anyway.
@@ -88,6 +90,11 @@ namespace OpenCSG {
             int          oldFramebufferID;
 
             bool         initialized;
+
+        private:
+            FrameBufferObject(const FrameBufferObject&);
+            FrameBufferObject& operator=(const FrameBufferObject&);
+
         };
 
     } // namespace OpenGL
