@@ -1,21 +1,20 @@
 // OpenCSG - library for image-based CSG rendering for OpenGL
-// Copyright (C) 2002-2014, Florian Kirsch,
+// Copyright (C) 2002-2022, Florian Kirsch,
 // Hasso-Plattner-Institute at the University of Potsdam, Germany
 //
-// This library is free software; you can redistribute it and/or 
-// modify it under the terms of the GNU General Public License, 
-// Version 2, as published by the Free Software Foundation.
-// As a special exception, you have permission to link this library
-// with the CGAL library and distribute executables.
+// This library is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-// This library is distributed in the hope that it will be useful, 
-// but WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+// This library is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110 - 1301 USA.
 
 //
 // renderGoldfeather.cpp
@@ -49,7 +48,8 @@ namespace OpenCSG {
 
         void GoldfeatherChannelManager::merge() {
 
-            setupProjectiveTexture();
+            bool isFixedFunction = true;
+            setupProjectiveTexture(isFixedFunction);
 
             glEnable(GL_ALPHA_TEST);
             glAlphaFunc(GL_GEQUAL, 0.5f); // accuracy issue with GL_EQUAL 1.0 on FX5600
@@ -91,7 +91,7 @@ namespace OpenCSG {
 
             scissor->disableScissor();
 
-            resetProjectiveTexture();
+            resetProjectiveTexture(isFixedFunction);
 
             clear();
         }
@@ -480,7 +480,9 @@ namespace OpenCSG {
             case OcclusionQuery:
                 if (renderOcclusionQueryGoldfeather(primitives))
                     break;  // success
-                            // else fall through (should not happen in practice due to the check for extensions of the caller)
+                // coming here should not happen in practice due to the check
+                // for extensions performed by the caller. Anyway, if it happens,
+                // fall through
             case NoDepthComplexitySampling:
                 renderGoldfeather(primitives);
                 break;
